import 'dart:convert';
import 'dart:io';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

import 'package:firebase_auth/firebase_auth.dart';
import 'package:http/http.dart' as http;
import 'package:wowit/common.dart';
import 'package:wowit/firebaseService.dart';
import 'package:wowit/methods.dart';
import 'package:wowit/sqlService.dart';

class SQLServiceToken {
  static const String _HOST = "https://wowit.xyz/";

  static const String _URI_GET_TOKEN = "2888DCF1-F630-4AEC-8E65-2AA1021B66BA";
  static const String _URL_GET_TOKEN = _HOST + _URI_GET_TOKEN;

  static const String _URI_API = "2888DCF1-4AEC-8E65-F630-2AA1021B66BA";
  static const String _URL_API = _HOST + _URI_API;

  static String _tokenApi;
 

  static Future _initToken(String phone, String uidPhone,
      {bool force = false}) async {
    try {
      if (force) {
        _tokenApi = null;
      }
      if (_tokenApi != null) return;
      var response = await http.post(Methods.getURI(_URL_GET_TOKEN), body: {
        "grant_type": "password",
        "username": utf8.fuse(base64).encode(phone),
        "password": utf8.fuse(base64).encode(uidPhone),
      });
      Map<String, dynamic> dsResult = json.decode(response.body);
      if (dsResult.containsKey('access_token')) {
        _tokenApi = dsResult['access_token'];
      }
    } catch (e) {
      print('SQLService.getToken with exception:  ' + e.toString());
    }
  }

  static Future<Map<String, List<Map<String, Object>>>> executeWithTokenAPI(
      String query,
      {Map<String, Object> params,
      bool tryExec = true}) async {
    if (FirebaseAuth.instance.currentUser == null) return {};
    await _initToken(FirebaseAuth.instance.currentUser.phoneNumber,
        FirebaseAuth.instance.currentUser.uid);
    params = SQLService.setupDefaultParameter(query, params);

    String jparamsStringToBase64 = utf8.fuse(base64).encode(jsonEncode(params));
    print(
        "- execute query: $query\n- jParam ${params.toString()}\n- jparamsStringToBase64: $jparamsStringToBase64");

    Map<
        String, // table namme
        List< // list row -> data tabl
            Map<String, Object> //data row
            >> retVal = {};
    try {
      var response = await _requestHttp({'a': jparamsStringToBase64});
      if (response.statusCode == 401) {
        await _initToken(FirebaseAuth.instance.currentUser.phoneNumber,
            FirebaseAuth.instance.currentUser.uid,
            force: true);
        response = await _requestHttp({'a': jparamsStringToBase64});
      }
      if (response.statusCode == 401) {
        //401 again -> Authorization has been denied for this request -> Logout
        await FirebaseService.logout();
        Get.offAll(() => Common.registerScreen ?? Container());
        return retVal;
      }

      if (response.statusCode != 200) return retVal;

      if (response.body == "") {
        print("====================[EMPTY RESPONSE]====================");
        return retVal;
      }
      //--> return data set
      Map<String, dynamic> dsResult = json.decode(response.body);
      dsResult.forEach((key, value) {
        List<dynamic> table = value;
        List<Map<String, Object>> rsTable = [];
        for (int i = 0; i < table.length; i++) {
          rsTable.add(table[i]);
        }
        retVal[key] = rsTable;
      });
    } catch (e) {
      print('SQLService.execute with exception:  ' + e.toString());
    }
    return retVal;
  }
 
  static Future<http.Response> _requestHttp(_body) async {
    return await http.post(Uri.parse(_URL_API),
        headers: <String, String>{
          HttpHeaders.contentTypeHeader: "application/json",
          HttpHeaders.authorizationHeader: "Bearer $_tokenApi",
        },
        body: _body);
  }
}
